import type { LaxPartial } from "@samual/lib";
export { minify } from "./minify";
export { postprocess } from "./postprocess";
export { preprocess } from "./preprocess";
export { transform } from "./transform";
export type ProcessOptions = {
    /** whether to minify the given code */
    minify: boolean;
    /** 11 a-z 0-9 characters */
    uniqueID: string;
    /** the user going to be hosting this script (or set to `true` if not yet known) */
    scriptUser: string | true;
    /** the name of this script (or set to `true` if not yet known) */
    scriptName: string | true;
    filePath: string;
    /** whether to mangle function and class names (defaults to `false`) */
    mangleNames: boolean;
    /**
     * when set to `true` forces use of quine cheats
     *
     * when set to `false` forces quine cheats not to be used
     *
     * when left unset or set to `undefined`, automatically uses or doesn't use quine cheats based on character count
     */
    forceQuineCheats: boolean;
};
/**
 * Minifies a given script
 *
 * @param code JavaScript or TypeScript code
 * @param options {@link ProcessOptions details}
 */
export declare const processScript: (code: string, { minify: shouldMinify, uniqueID, scriptUser, scriptName, filePath, mangleNames, forceQuineCheats }?: LaxPartial<ProcessOptions>) => Promise<{
    script: string;
    warnings: {
        message: string;
        line: number;
    }[];
}>;
export default processScript;
