import type { LaxPartial } from "@samual/lib";
import type { Info } from ".";
export type PushOptions = {
    /** whether to do the minify step (defaults to `true`) */
    minify: boolean;
    /** whether to mangle function and class names (defaults to `false`) */
    mangleNames: boolean;
    /**
     * array of scripts in the format `foo.bar`
     *
     * also accepts wild card (`*`) e.g. `*.bar` or `foo.*`
     *
     * pushes everything by default (`*.*`)
     */
    scripts: string[];
    /** callback called on script push */
    onPush: (info: Info) => void;
    /**
     * when set to `true` forces use of quine cheats
     *
     * when set to `false` forces quine cheats not to be used
     *
     * when left unset or set to `undefined`, automatically uses or doesn't use quine cheats based on character count
     */
    forceQuineCheats: boolean;
};
/**
 * Push scripts from a source directory to the hackmud directory.
 *
 * Pushes files directly in the source folder to all users
 * @param sourceDirectory directory containing source code
 * @param hackmudDirectory directory created by hackmud containing user data including scripts
 * @param options {@link PushOptions details}
 * @returns array of info on pushed scripts
 */
export declare const push: (sourceDirectory: string, hackmudDirectory: string, { scripts, onPush, minify, mangleNames, forceQuineCheats }?: LaxPartial<PushOptions>) => Promise<Info[]>;
export default push;
