import { readdir, readFile, stat, writeFile } from 'fs/promises';
import { extname, basename, resolve } from 'path';

const syncMacros = async hackmudPath => {
  const files = await readdir(hackmudPath, {
    withFileTypes: true
  });
  const macros = new Map();
  const users = [];
  await Promise.all(files.map(async file => {
    if (!file.isFile()) return;
    switch (extname(file.name)) {
      case `.macros`:
        {
          const [lines, date] = await Promise.all([readFile(resolve(hackmudPath, file.name), {
            encoding: `utf-8`
          }).then(file => file.split(`\n`)), stat(resolve(hackmudPath, file.name)).then(({
            mtime
          }) => mtime)]);
          for (let index = 0; index < lines.length / 2 - 1; index++) {
            const macroName = lines[index * 2];
            const currentMacro = macros.get(macroName);
            if (!currentMacro || date > currentMacro.date) macros.set(macroName, {
              date,
              macro: lines[index * 2 + 1]
            });
          }
        }
        break;
      case `.key`:
        {
          users.push(basename(file.name, `.key`));
        }
        break;
    }
  }));
  let macroFile = ``;
  let macrosSynced = 0;
  for (const [name, {
    macro
  }] of [...macros].sort(([a], [b]) => (a > b) - (a < b))) {
    if (macro[0] != macro[0].toLowerCase()) continue;
    macroFile += `${name}\n${macro}\n`;
    macrosSynced++;
  }
  for (const user of users) writeFile(resolve(hackmudPath, `${user}.macros`), macroFile);
  return {
    macrosSynced,
    usersSynced: users.length
  };
};

export { syncMacros as default, syncMacros };
